#ifndef __CAM_H__
#define __CAM_H__

// cam.h -- handles cam server

#include "btkcam.h"

#ifdef WIN32
#include <process.h>
#include "win32.h"
#else
#include <pthread.h>
#include <sys/socket.h>
#endif


#define FILTER_TIMEOUT      (604800 * 1000)
#define FILTER_PEEK_TIMEOUT (1 * 1000)

// ---------------------------------------------------------

#define CAID_DESCRIPTIONS_COUNT 0x49
typedef enum
{
	SLEEPING,
	DATA_AVAILABLE,
	DISCONNECTED
} service_status_t;

typedef struct ca_descriptor_info
{
	unsigned short int ca_descriptor;
	unsigned short int ecm_pid;
	unsigned char fta;
	unsigned int provid;
} ca_descriptor_info_t;

typedef struct pid_section
{
	unsigned char type;
	char type_description[13];
	unsigned short int pid;
	ca_descriptor_info ca_info[16];
	unsigned short int ca_info_count;
} pid_section_t;

class service
{
	public:
	service(int accepted_message_fd);
	~service(void);
	
	unsigned int log_trans;
	
	service* previous;
	service* next;
	
	service_status_t status;
	
	unsigned short 	int caid;
	unsigned 	int provider_id;
	int		IRDkeynr;
	int		IRDchid;
	
	//service_related
	unsigned short int sid;
	unsigned int dvb_namespace;
	unsigned short int tsid;
	unsigned short int onid;
	unsigned short int pmt_pid;
	
	pid_section_t pid_sections[16];
	unsigned short int pid_sections_count;
	
	ca_descriptor_info root_ca_info[16];
	unsigned short int root_ca_info_count;

	ca_descriptor_info ecm_ca_info[16];
	unsigned short int ecm_ca_info_count;
		
	int descramble_mask;	
	int descramble_index;
	int demux_index;
	
	// ecm related
	unsigned char ecm_data_raw[512];
	unsigned char ecm_data_original_header[10];
	unsigned char ecm_data[512];
	unsigned int ecm_data_length;
	int last_ecm_table;	
	unsigned short int ecm_table;
	unsigned short int ecm_pid;
	int ecm_demux_fd;
	int failed_ecm_decryption_count;
	pthread_t ecm_thread;
		

	
	int message_fd;
	unsigned char message[1024];
	int message_length;
	
	int decoder_running;
	
	unsigned char cw_0_plain[8];
	unsigned char cw_1_plain[8];
	unsigned char last_cw_0_plain[8];
	unsigned char last_cw_1_plain[8];
	
	// ----------------------------------------------------------
	int	ActB;
	int	Algo;
	char 	btkcmd[1024];
	char 	keyret[100];

	unsigned char cw_0[8];
	unsigned char cw_1[8];
	unsigned char last_cw_0[8];
	unsigned char last_cw_1[8];
	

	//-------- static member & function declarations -----------------------
	static service* service_root;
	static int listener_fd;
	static service_status_t listener_status;
	static pthread_mutex_t cardserver_lock;
	static int initialize( void );
	static void finalize( void );
	
	static void *process_ecm_thread( void *handler );
	
	static void add( service* new_service );
	static void remove( service* remove_service );
	static void stop_all_decoders(void);
	static int get_free_descramble_index(void);
	
	static int count( void );
	static int poll_services( void );
	static bool accept_service_request( void );
	static void remove_inactive( void );
	static service* get_service_by_fd( int fd );
	static bool fd_is_listener( int fd );

	static void refresh_ca_pids( void );

	
	// non-static functions declarations
	int fd_peek( int fd, unsigned int wait );	
	
	service* ca_pid_used_by_another_service( unsigned short int ca_device_index, unsigned short int pid );
	void set_ca_pids( void );
	void clear_ca_pids( void );
		
	void process_ecm_thread_cleanup(void);	
	bool checkecms(void);	
	bool checkdcws(void);
	bool localdcw(void);

	int process_message( void );
	bool parse_message(unsigned char *data, int length);
	unsigned char *parse_message_info_section(unsigned char *data);
	unsigned char *parse_message_pid_section(unsigned char *data);
	
	void start_decoder( void );
	void stop_decoder( void );
};

#endif
